#ifndef MESH_H
#define MESH_H
/*
	Animadead: A Skeletal Animation Library
	Copyright (C) 2005 John C. Butterfield

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

	John C. Butterfield
	johnb003@hotmail.com
*/

#include <vector>
#include <string>
#include <map>
//#include "animation.h"
#include "matrixmath.h"

namespace ad
{
    // this stores the influence for a particular bone on a vertex
    struct Influence
    {
        unsigned int bone;
        float weight;
    };

    // this is is all the information about each vertex
    struct SuperVertex							
	{
		float x,y,z,a,b,c,u,v;					// x,y,z is the point, a,b,c is the normal, and u,v are the texture coordinates
		Influence *weights;
		unsigned int numInfluences;

		SuperVertex();
		~SuperVertex();
	};

	// this stores the polygon faces
	struct Face
	{
		unsigned int numIndices;
		unsigned int *indices;					// refers to the index of the vertex

		Face();
		~Face();
	};

	// this is used by the model loader to call a user defined texture loader.
	// the user can return information such a texture ID in "userdata"
	class UserFunction
	{
	public:
		virtual void operator()(std::string filename, void *&userdata) {}
	};

    // a surface is a mesh that contains a uv map, a texture and the mesh data.
    // it's abstracted from a 'Mesh' to allow different textures within the same model for more detail
	struct Surface								
	{
		unsigned int nameLength;
		char *texturename;						// every time a texture filename is read from the file, it passes it and the user data to the user function
		unsigned int numVerts;
		SuperVertex *superVerts;
		unsigned int numFaces;
		void *userData;							// this can be used to store information about the surface. It is called when the user function is called
		Face *faces;

		Surface();
		~Surface();
	};

	struct Mesh
	{
		unsigned int numBones;					// number of bones that affect this mesh (if 0, mesh is static)
		unsigned int *bones;					// the list of influences (if any) are the indicies of the bones relivant to this model
		Matrix *inverseBindPose;				// array of bones transforms (inverse bind transform)
		unsigned int numSurfaces;
		Surface *surfaces;

		Mesh(std::string filename, UserFunction *loader = 0); // Loads a mesh from an .adm file
		Mesh();
		~Mesh();
	};

	class MeshCacher							// loads a mesh and keeps track of the instances of previously loaded meshes.
	{
		MeshCacher();							// singleton, use MeshCacher::instance() to use it.
		static MeshCacher *inst;
		std::map<std::string, Mesh *> cache;

	public:
		~MeshCacher();
		static MeshCacher *instance();
		void release();
		Mesh *GetMesh(std::string filename, UserFunction *fp = 0);	// returns instance of loaded .adm file, or loads it if it isn't already.
	};
}

#endif // MESH_H
